<?php

namespace Drupal\commerce_payrexx_integration\PluginForm;

use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Payrexx\Models\Request\Gateway;
use Payrexx\Payrexx;
use Payrexx\PayrexxException;

/**
 * Redirect form for off-site Payrexx payment.
 */
class RedirectCheckoutForm extends BasePaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $configuration = $this->getConfiguration();

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    $order = Order::load($this->entity->getOrderId());

    $payment_method = $order->getData('selected_payment_method') ? $order->getData('selected_payment_method') === 'any' ? [] : [$order->getData('selected_payment_method')] : [];

    $payrexx = new Payrexx($configuration['instance_name'], $configuration['secret']);
    $gateway = new Gateway();

    $gateway->setAmount(($payment->getAmount()->getNumber() + ($payment->getAmount()->getNumber() * $configuration['fee'] / 100)) * 100);
    $gateway->setCurrency($payment->getAmount()->getCurrencyCode());
    $gateway->setVatRate($configuration['vat']);
    $gateway->setSuccessRedirectUrl($form['#return_url']);
    $gateway->setCancelRedirectUrl($form['#cancel_url']);
    $gateway->setFailedRedirectUrl($form['#cancel_url']);
    $gateway->setPm($payment_method);
    $gateway->setPreAuthorization(FALSE);
    $gateway->setReservation(FALSE);
    $gateway->setReferenceId($this->createOrderId());
    if ($configuration['send_basket'] == '1') {
      $gateway->setBasket($this->prepareOrderItems($order));
    }
    try {
      $response = $payrexx->create($gateway);
      $order->setData('payrexx_gateway_id', $response->getId());
      $order->save();
    }
    catch (PayrexxException $e) {
      print $e->getMessage();
      exit;
    }

    return $this->buildRedirectForm(
      $form,
      $form_state,
      $response->getLink(),
      [],
      PaymentOffsiteForm::REDIRECT_POST
    );
  }

  /**
   * Build the order id taking order prefix into account.
   *
   * @return string
   *   Return order number.
   */
  private function createOrderId() {
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    $order_id = $payment->getOrderId();

    // Order number must be at least 4 characters.
    // If not, QuickPay will reject the request.
    if (strlen($order_id) < 4) {
      $order_id = substr('000' . $order_id, -4);
    }

    return $order_id;
  }

  /**
   * Returns the gateway configuration.
   *
   * @return array
   *   Returns the configuration array.
   */
  private function getConfiguration() {
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    return $payment_gateway_plugin->getConfiguration();
  }

  /**
   * Prepares the order items for Payrexx gateway.
   *
   * @param \Drupal\commerce_order\Entity\Order $order
   *   The commerce order entity.
   *
   * @return array
   *   Array of order products.
   */
  private function prepareOrderItems(Order $order) {
    $order_items = [];
    if (count($order->getItems()) > 0) {
      foreach ($order->getItems() as $item) {
        $item_price = $item->getAdjustedUnitPrice();
        $order_items[] = [
          'name' => [
            1 => $item->getTitle(),
          ],
          'quantity' => (int) $item->getQuantity(),
          'amount' => $item_price->getNumber() * 100,
        ];
      }
    }
    $order_adjustments = $order->getAdjustments();
    if (count($order_adjustments) > 0) {
      foreach ($order_adjustments as $order_adjustment) {
        if (floatval($order_adjustment->getAmount()->getNumber()) != 0) {
          $order_items[] = [
            'name' => [
              1 => $order_adjustment->getLabel(),
            ],
            'quantity' => 1,
            'amount' => floatval(round($order_adjustment->getAmount()->getNumber(), 2) * 100),
          ];
        }
      }
    }
    return $order_items;
  }

}
